@@ -116,7 +116,7 @@ class Note(object):
         self.pos = pos or (0, 0)
         self.start = start or (0, 0)
         self.wobble = wobble or 0.5
-        self.reserved = reserved or [0, 0, 0]
+        self.reserved = reserved or [-2, 0, 0]
         assert len(self.reserved) == 3
 
     @classmethod
@@ -160,7 +160,7 @@ class NoteDef(object):
     def __init__(self, *notes):
         super(NoteDef, self).__init__()
         self.notes = notes
-        assert 0 < len(self.notes) <= 4
+        assert 0 <= len(self.notes) <= 4
         self.__struct = struct.Struct('<I144x28x')
         self.__struct_note = struct.Struct('<I5fiII')
         assert self.__struct.size == 0xB0
@@ -179,6 +179,7 @@ class NoteDef(object):
                                          note.wobble,
                                          *note.reserved)
 
+        assert len(buffer) == 0xB0
         return buffer
 
     @staticmethod
@@ -200,9 +201,9 @@ class NoteDef(object):
 
     @classmethod
     def random(cls, **kwargs):
-        min_ = min(max(kwargs.pop('min') or 1, 1), 4)
-        max_ = max(min(kwargs.pop('max') or 1, 4), 1)
-        duplicates = kwargs.pop('duplicates') or True
+        min_ = min(max(kwargs.pop('min', 1), 1), 4)
+        max_ = max(min(kwargs.pop('max', 4), 4), 1)
+        duplicates = kwargs.pop('duplicates', True)
 
         buttons = random.choices(range(min_, max_ + 1), [i * 0.25 for i in range(min_, max_ + 1)][::-1])[0]
         notes = []
@@ -308,14 +309,14 @@ class FutureTone(object):
         definitions = await self.get_score_definitions()
         return definitions[type_]
 
-    async def set_score_definitions(self, *score_definitions: ScoreDefinition):
+    async def set_score_definitions(self, *score_definitions: ScoreDefinition | tuple[int, bool, int, bool, int]):
         structure = struct.Struct('<5I')
         types_ = list(NoteResultType)
 
         assert len(score_definitions) == len(types_)
 
-        for i, type_ in enumerate(types_):
-            data = tuple(score_definitions[i])
+        for i, definition in enumerate(score_definitions):
+            data = tuple(definition)
             await self.ps4debug.write_struct(self.pid, self.base + 0x8ACE30 + i * structure.size, structure, *data)
 
     async def set_score_definition(self, type_: NoteResultType, value: ScoreDefinition):
@@ -375,21 +376,48 @@ class FutureTone(object):
 
     async def get_chart(self) -> list[NoteDef] | None:
         start = await self.ps4debug.read_uint64(self.pid, self.base + 0xC687708 + 0x2C600)
+        if start == 0:
+            return
 
+        chart = []
+        async for note_def in self.enumerate_chart():
+            chart.append(note_def)
+
+        return chart
+
+    async def enumerate_chart(self):
+        start = await self.ps4debug.read_uint64(self.pid, self.base + 0xC687708 + 0x2C600)
         if start == 0:
             return
 
         end = await self.ps4debug.read_uint64(self.pid, self.base + 0xC687708 + 0x2C608)
-        count = (end - start) // 0xB0
+        size = end - start
+        data = await self.ps4debug.read_memory(self.pid, start, size)
 
-        chart = []
-        for i in range(count):
-            address = start + i * 0xB0
-            data = await self.ps4debug.read_memory(self.pid, address, 0xB0)
-            note_def = NoteDef.from_bytes(data)
-            chart.append(note_def)
+        for i in range(0, size, 0xB0):
+            note_def = NoteDef.from_bytes(data[i:i+0xB0])
+            yield note_def
 
-        return chart
+    async def set_chart(self, *note_defs: NoteDef):
+        start = await self.ps4debug.read_uint64(self.pid, self.base + 0xC687708 + 0x2C600)
+        if start == 0:
+            return
+
+        # TODO create parameter # TODO doesn't work
+        offset = 0
+        start += offset * 0xB0
+        end = await self.ps4debug.read_uint64(self.pid, self.base + 0xC687708 + 0x2C608)
+        size = end - start
+        count = size // 0xB0
+
+        if len(note_defs) > count:
+            return
+
+        data = bytearray()
+        for i, note_def in enumerate(note_defs):
+            data.extend(note_def.to_bytes())
+        assert len(data) == size
+        await self.ps4debug.write_memory(self.pid, start, data)
 
     async def get_chart_size(self) -> int | None:
         start = await self.ps4debug.read_uint64(self.pid, self.base + 0xC687708 + 0x2C600)
