@@ -195,16 +195,26 @@ class FutureToneIntegrationBot(commands.Bot):
         await self.pubsub.subscribe_topics([topic(token)[id_] for topic in topics])
 
     async def apply_config(self, config: Settings):
+        if config.global_.verbose:
+            typer.echo(f'Applying {typer.style("Max life", fg="green")}...')
         await self.ft.set_max_life(config.game.max_life)
+        if config.global_.verbose:
+            typer.echo(f'Applying {typer.style("Starting life", fg="green")}...')
         await self.ft.set_starting_life(config.game.start_life)
 
+        if config.global_.verbose:
+            typer.echo(f'Applying {typer.style("lyric settings", fg="green")}...')
         if config.game.disable_lyrics:
             await self.ft.disable_lyrics(clear=True)
         else:
             await self.ft.enable_lyrics(clear=False)
 
+        if config.global_.verbose:
+            typer.echo(f'Applying {typer.style("max volume", fg="green")}...')
         await self.ft.set_max_volume(config.game.max_volume)
 
+        if config.global_.verbose:
+            typer.echo(f'Applying {typer.style("1.00 patch settings", fg="green")}...')
         if config.game.unpatch_holds:
             await self.ft.disable_hold_invalidation()
         else:
@@ -216,7 +226,35 @@ class FutureToneIntegrationBot(commands.Bot):
         else:
             await self.ft.enable_score_invalidation()
 
+        if config.global_.verbose:
+            typer.echo(f'Applying {typer.style("score definitions", fg="green")}...')
+        await self.ft.set_score_definitions(
+            config.game.cool,
+            config.game.fine,
+            config.game.safe,
+            config.game.bad,
+            config.game.cool_wrong,
+            config.game.fine_wrong,
+            config.game.safe_wrong,
+            config.game.bad_wrong,
+            config.game.miss,
+            config.game.cool_double,
+            config.game.fine_double,
+            config.game.safe_double,
+            config.game.bad_double,
+            config.game.cool_triple,
+            config.game.fine_triple,
+            config.game.safe_triple,
+            config.game.bad_triple,
+            config.game.cool_quad,
+            config.game.fine_quad,
+            config.game.safe_quad,
+            config.game.bad_quad,
+        )
+
         # Redemption LUT
+        if config.global_.verbose:
+            typer.echo(f'Registering {typer.style("custom rewards", fg="green")}...')
         self.redemptions = {
             config.spawn_random.title: (config.spawn_random, self.redeem_spawn_random),
             config.spawn.title: (config.spawn, self.redeem_spawn),
@@ -488,6 +526,21 @@ class FutureToneIntegrationBot(commands.Bot):
             await ctx.reply('There are some errors in the configuration file.')
             typer.echo(ex, err=True)
 
+    @commands.command(name='sendpayload', aliases=['payload'])
+    @requires_mod(exempt=['0jay0'])
+    async def send_payload(self, ctx: commands.Context):
+        """Sends the required PlayStation4 payload to the system."""
+        if self.config.global_.verbose:
+            typer.echo(f'Payload sending issued by {typer.style(ctx.author.name, fg="green")}')
+
+        ip_address = self.config.global_.ip
+        if ip_address is None:
+            typer.secho(f'Cannot send payload when IP address is not specified explicitly.', fg='red', err=True)
+            return
+
+        port = int(ctx.view.words[1]) if len(ctx.view.words) > 0 else 9020
+        await self.ft.ps4debug.send_ps4debug(ip_address, port)
+
     @commands.command()
     async def laundry(self, ctx: commands.Context):
         """Time to do laundry."""
@@ -604,25 +657,40 @@ class FutureToneIntegrationBot(commands.Bot):
         """Spawns a random note."""
         await self.ft.spawn_note(NoteDef.random())
 
-    @commands.command(name='crashthegame')
+    @commands.command(name='secret')
     @requires_mod(exempt=['0jay0'])
-    async def dont_do_this(self, ctx: commands.Context):
-        """Really, don't."""
-        note_types = list(NoteType)
-        note_types = {t.value: random.choice(note_types) for t in note_types}
-
-        print(note_types)
+    async def secret(self, ctx: commands.Context):
+        """Don't use it :)"""
+        #note_types = list(NoteType)
+        #note_types = {t.value: random.choice(note_types) for t in note_types}
+        #print(note_types)
 
         size = await self.ft.get_chart_size()
         while not size:
-            await asyncio.sleep(0.5)
+            await asyncio.sleep(0.25)
             size = await self.ft.get_chart_size()
 
         typer.echo('Starting...')
-        for i in range(size):
-            note_def = await self.ft.get_chart_note(i)
+        offset_per_note = 25 / size
+        note_change_at = size // 2
+        note_change_chance_per_note = 0.00075
+        chart = []
+
+        async for note_def in self.ft.enumerate_chart():
+            i = len(chart)
+
             for note in note_def.notes:
-                note.button = note_types[note.button.value]
-            await self.ft.set_chart_note(i, note_def)
-            typer.echo(f'{i}/{size}')
+                offset_x = (random.random() * 2 - 1) * offset_per_note * i
+                offset_y = (random.random() * 2 - 1) * offset_per_note * i
+                x, y = note.pos
+                note.pos = (x + offset_x, y + offset_y)
+
+                if i >= note_change_at:
+                    chance = note_change_chance_per_note * (i - note_change_at)
+                    if random.random() <= chance:
+                        new_button = random.choice(list(NoteType))
+                        note.button = new_button
+            chart.append(note_def)
+
+        await self.ft.set_chart(*chart)
         typer.echo('Done...')
